##############################################################################
#
# Copyright (c) 2010 ViFiB SARL and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


import os
from slapos.tests.libnetworkcachemixin import LibNetworkCacheMixin
from slapos.signature import parseArgument


class GenerateSignatureScriptTest(LibNetworkCacheMixin):
  """ Class which must test the signature.py script. """

  def test_parse_argument_with_empty_list(self):
    """
      If the argument list is empty, then the parseArgument method should
    return a dictionary with default argument values.
    """
    default_dict = {'organization_name': 'Default Company Ltd',
                    'state_name': 'Default Province',
                    'organization_unit_name': '',
                    'common_name': '',
                    'country': 'XX',
                    'locality_name': 'Default City',
                    'signature_private_key_file': 'private.pem',
                    'signature_certificate_file': 'public.pem',
                    'email': ''}
    self.assertEquals(default_dict, parseArgument())

  def test_parse_argument(self):
    """
      Check if the argument is properly set.
    """
    size_argument_list = len(self.signature_creation_argument_list) / 2
    size_option_dict = len(self.option_dict)
    self.assertEquals(size_argument_list, size_option_dict,
        "Argument list should have the same size of option dict.")

    # Assert if the values are equals.
    for value in self.option_dict.values():
      self.assertTrue(value in self.signature_creation_argument_list,\
          '%s is not in %s.' % (value, self.signature_creation_argument_list))

  def test_key_and_certificate_file_creation(self):
    """
      Check if key file and the certificate file are being created correctly.
    """
    self.assertTrue(os.path.exists(self.signature_certificate_file))
    self.assertTrue(os.path.exists(self.signature_private_key_file))
